predictor_url = 'http://dlib.net/files/shape_predictor_68_face_landmarks.dat.bz2'
predictor_file = '../weights/shape_predictor_68_face_landmarks.dat'
p2v_model_gdrive_id = '1op5_zyH4CWm_JFDdCUPZM4X-A045ETex'
sample_image = '../examples/sample.jpg'
